import type { SanityImageCrop, SanityImageHotspot } from './types';
/**
 * Default crop (equals to "whole image")
 */
export declare const DEFAULT_CROP: Readonly<SanityImageCrop>;
/**
 * Default hotspot (equals to horizontal/vertical center, full size of image)
 */
export declare const DEFAULT_HOTSPOT: Readonly<SanityImageHotspot>;
/**
 * Returns cloned version of the default crop (prevents accidental mutations)
 *
 * @returns Default image crop object
 */
export declare const getDefaultCrop: () => SanityImageCrop;
/**
 * Returns cloned version of the default hotspot (prevents accidental mutations)
 *
 * @returns Default image hotspot object
 */
export declare const getDefaultHotspot: () => SanityImageHotspot;
/**
 * Returns whether or not the passed crop has the default values for a crop region
 *
 * @param crop The crop to return whether or not is the default crop
 * @returns True if passed crop matches default, false otherwise
 */
export declare const isDefaultCrop: (crop: SanityImageCrop) => boolean;
/**
 * Returns whether or not the passed hotspot has the default values for a hotspot region
 *
 * @param hotspot The hotspot to return whether or not is the default hotspot
 * @returns True if passed hotspot matches default, false otherwise
 */
export declare const isDefaultHotspot: (hotspot: SanityImageHotspot) => boolean;
